/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include "oirSupport.h"
#include "fitsio.h"
#include "imaging_data.h"
#include "imaging_fdata.h"
#include "minrtsFrameData.h"

/**********************************************************
 * Compute average target-sky frame from a chopping sequence
 * Usage:
 *    unChop  infile outfile [nDrop]
 *        infile is blank separated, quoted list of a sequence
 *            of IMAGING_DATA tables with a chopping sequence
 *        outfile is IMAGING_FDATA table with result.
 *        nDrop [optional] is number of frames to drop after 
 *            state change, default 2
 ************************************************************* */

/* Local prototypes   */

int main (int argc, char** argv) {

#define MAXREGION 12 /* Maximum number of regions */
   char **fileList;
   int nFiles;
   int i,ierr;
   char outFileName[130];
   char *bang="!";
   fitsfile *inFile;
   fitsfile *outFile = NULL;
   imaging_fdata_table_def *odata;
   imaging_detector_table_def *DetectorDef;
   imaging_fdata_row *outData;
   int iReg;
   long npix, iPix, naxes[2] = {0,0};
   minrtsFrameData *onFData, *offFData, *difFData;
   int  nkeys = 11; /* Header keywords to copy */
   char *keys[] = {"HISTORY", "DATE-OBS", "MJD-OBS","ARCFILE ","TELESCOP",
		  "INSTRUME","ORIGIN", "EQUINOX", "RADECSYS", "UTC", "LST"};

   int nDrop; /* number of frames to drop after state change */

   ierr = 0;
   /*    see if nDrop is specified externally*/
   nDrop = 2;
   if (argc >= 4) sscanf(argv[3], "%i", &nDrop);
   printf("nDrop = %i\n", nDrop);

   /*        
	     break up file list and print what you find 
	     then read and unchop files
   */
   fileList = oirStrSplit(argv[1], &nFiles);

   /*
     open first file
   */
   i = fits_open_file(&inFile, fileList[0], READONLY, &ierr) ;
   if (ierr != 0) {
   fprintf(stderr,"cfitsio error %i opening file %s\n", ierr, fileList[0]);
   fits_report_error(stderr, ierr);
   return 1;
   }
   ierr = 0;
   /*
     get detector section
   */
   DetectorDef = read_imaging_detector_table(inFile, &ierr);
   if (ierr !=0) {
     fprintf(stderr,"error %i opening detector table in %s\n", 
	     ierr, fileList[0]);
     fits_report_error(stderr, ierr);
     return 1;
   }
   /* 
      create FrameDatas for on, off-source, difference
   */
   onFData  = minrtsFrameDataMake(DetectorDef);
   offFData = minrtsFrameDataMake(DetectorDef);
   difFData = minrtsFrameDataMake(DetectorDef);

   i = fits_close_file(inFile, &ierr);
   
   /* 
      read data 
   */
   minrtsFrameDataUnChop(fileList, nFiles, nDrop,
			 onFData, offFData, &ierr);
   /*
     subtract sky
   */
   minrtsFrameDataSub (difFData, onFData, offFData);
   
   /*
     prepare output file
   */
   strcpy(outFileName, bang);
   strcat(outFileName, argv[2]);
   
   fits_create_file (&outFile, outFileName, &ierr);
   if (ierr !=0) {
     fprintf (stderr, "error opening fits output %s %i\n",argv[2], ierr);
     fits_report_error(stderr, ierr);
     return 1;
   }
   

   /* Copy header keywords from first file */
   /*  open input  */
   i = fits_open_file(&inFile, fileList[0], READONLY, &ierr) ;
   /* Dummy image header to keep cfitsio happy */
   fits_write_imghdr(outFile, 8, 2, naxes, &ierr);
   /* Copy keywords */
   copy_header_keywords (inFile, outFile, nkeys, keys, &ierr);
   i = fits_close_file(inFile, &ierr);
   if (ierr != 0) {
     fprintf(stderr,"cfitsio error %i copying header keywords\n", ierr);
     fits_report_error(stderr, ierr);
     /* This shouldn't really be fatal */
     ierr = 0;
   }


   /* create output table */
   odata = make_imaging_fdata_table_def (outFile, DetectorDef, 1, &ierr);
   if (ierr !=0) {
     fprintf (stderr, "error creating data definition\n");
     fits_report_error(stderr, ierr);
     return 1;
   }
   
   /*
     create and initialize row 
   */
   outData = make_imaging_fdata_row(odata);
   outData->frame = 1;
   for (iReg=0; iReg<onFData->nregion; iReg++) {
     outData->targtype[iReg] = 'S';
   }
   
   /* 
      copy to output row 
   */
   for (iReg=0; iReg<onFData->nregion; iReg++) {
     npix = onFData->nx[iReg]*onFData->ny[iReg];
     for (iPix=0; iPix<npix; iPix++) outData->data[iReg][iPix] = difFData->data[iReg][iPix];
   } /* end region loop */
   
   /*   
	Write output and close
   */
   i = write_imaging_fdata_table(outFile, odata, outData, 1, &ierr);
   i = close_imaging_fdata_table(outFile, odata, &ierr);
   /*
     Cleanup
   */
   kill_imaging_fdata_row(outData);
   minrtsFrameDataKill(onFData);
   minrtsFrameDataKill(offFData);
   minrtsFrameDataKill(difFData);
   
   return 0;
} /* end main */
